/*
    Copyright 2004-2011 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

typedef unsigned char   u8;
typedef unsigned int    u32;



#define VER     "0.2"



u8 *get_chars(u8 *str, int *chars);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *inz,
            *outz;
    u32     num;
    int     len,
            chars,
            i,
            j;
    u8      buff[1024],
            *in,
            *out,
            *p,
            *c;

    setbuf(stdout, NULL);

    fputs("\n"
        "Rmchar "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stderr);

    if (argc < 4) {
        printf("\n"
            "Usage: %s <input> <output> <hex_chars>\n"
            "\n"
            " hex_chars can be one or more chars separated by comma like:\n"
            "   0x00 or 0x01,0x61,0x63\n"
            "\n", argv[0]);
        exit(1);
    }

    in  = argv[1];
    out = argv[2];

    fprintf(stderr, "- open input: %s\n", in);
    if(!strcmp(in, "-")) {
        inz = stdin;
    } else {
        inz = fopen(in, "rb");
    }
    if(!inz) std_err();

    fprintf(stderr, "- open output: %s\n", out);
    if(!strcmp(out, "-")) {
        outz = stdout;
    } else {
        outz = fopen(out, "wb");
    }
    if(!outz) std_err();

    c = get_chars(argv[3], &chars);
    fprintf(stderr, "- list of bytes to remove:");
    for(i = 0; i < chars; i++) {
        fprintf(stderr, " %02x", c[i]);
    }
    fprintf(stderr, "\n");

    num = 0;
    while((len = fread(buff, 1, sizeof(buff), inz)) > 0) {
        p = buff;
        for(i = 0; i < len; i++) {
            for(j = 0; j < chars; j++) {
                if(buff[i] == c[j]) break;
            }
            if(j < chars) {
                num++;
            } else {
                *p++ = buff[i];
            }
        }

        len = p - buff;
        if(fwrite(buff, 1, len, outz) != len) {
            fprintf(stderr, "\nError: write error\n");
            exit(1);
        }
    }

    if(inz  != stdin)  fclose(inz);
    if(outz != stdout) fclose(outz);
    fprintf(stderr, "- %u bytes erased\n\n", num);
    return(0);
}



u8 *get_chars(u8 *str, int *chars) {
    int     c,
            n;
    u8      *ret = NULL,
            *p,
            *l;

    n = 0;
    for(p = str; *p; p = l + 1) {
        for(l = p; *l; l++) {
            if(!isalnum(*l)) break;
        }
        sscanf(p, "%x", &c);
        ret = realloc(ret, n + 1);
        if(!ret) std_err();
        ret[n] = c;
        n++;
        if(!*l) break;
    }
    *chars = n;
    return(ret);
}



void std_err(void) {
    perror("\nError");
    exit(1);
}

